import java.io.*;
import javax.bluetooth.*;
import javax.microedition.io.*;

/**Clase que implementa todo lo necesario para la coneccion por bluetooth*/
public class ClientServer  implements DiscoveryListener{
    
    UUID RFCOMM_UUID = new UUID(0x0003);
    private StreamConnection strmConnection = null;
    private LocalDevice localDevice = null;
    private InputStream inputStrm = null;
    private OutputStream outputStrm = null;
    private StreamConnectionNotifier  StrmNotifier = null;

    /**indica si es servidor o no*/
    public boolean IsServer = false;
    /**indica si se encontro un servidor*/
    public boolean serverFound = false;
    /**indica cuando partir el servidor*/
    public boolean initServer = false;
    /**indica cuando partir el cleinte*/
    public boolean initClient = false;
    
    private static String strUrl;
    private DiscoveryAgent discovAgent = null;

    /**Constructor de la clase ClientServer
     @param isServer indica si es un servidor, si no, es un cleinte*/
    public ClientServer(boolean isServer)
    {
        IsServer = isServer;

        if(IsServer)
        {
            InitServer();//si es un servidor comienza el servidor
        }
        else
        {
            InitClient();//sino, comienza el cliente
        }
    }
    
    private void InitServer()
    {
        strUrl= "btspp://localhost:" + RFCOMM_UUID + ";name=rfcommtest;authorize=true";//setea la url del dispositivo

        try
        {
            localDevice = LocalDevice.getLocalDevice();
            localDevice.setDiscoverable(DiscoveryAgent.GIAC);//setea el dispositivo como descubrible
            StrmNotifier = (StreamConnectionNotifier)Connector.open(strUrl);
            strmConnection = StrmNotifier.acceptAndOpen();
            initServer = true;//inicia el servidor
            outputStrm = strmConnection.openOutputStream();//abre los streams
            inputStrm = strmConnection.openInputStream();
        }
        catch (BluetoothStateException e)
        {
            System.err.println( "BluetoothStateException: " + e.getMessage() );
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        catch(Exception e)
        {
            System.err.println( "Exception: " + e.getMessage() );
        }

    }
    
    private void InitClient()
    {
         SearchAvailDevices();
    }

    /**Busca telefonos disponibles*/
    public void SearchAvailDevices()
    {
        try
        {
            
            localDevice = LocalDevice.getLocalDevice();//obtiene el dispositivo local

            discovAgent=  localDevice.getDiscoveryAgent();//obtiene el discovery agent

            discovAgent.startInquiry(DiscoveryAgent.GIAC,this);
        }
        catch (BluetoothStateException ex)
        {
            System.out.println("Problem in searching the blue tooth devices");
            ex.printStackTrace();
        }
    }
    
    /**Envia un numero entero
     @param a numero entero a enviar*/
    public void SendInt(int a)
    {
        if((initClient) || (initServer) )
        {
            try
            {
                outputStrm.write(a);//escribe al Stream de salida
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }
        }
    }

    /**Recibe un numero entero*/
    public int RecieveInt()
    {
        int data=0;
        try
        {
            data = inputStrm.read();//recibe del stream de entrada
        }
        catch (IOException e)
        {
            System.err.println(e);
        }
        return data;
    }

    /**Mensaje de debugeo cuando la investigacion se completo*/
    public void inquiryCompleted(int discType)
    {
        System.out.println("InquiryCompleted");
    }

    /**llamado cuando el servicio de llamada fue completado*/
    public void serviceSearchCompleted(int transID, int respCode)
    {
        if(serverFound)//cuando se encuentra un servidor
        {
            try
            {   //deja la comunicacion partir
                strmConnection = (StreamConnection) Connector.open(strUrl);

                outputStrm = strmConnection.openOutputStream();
                inputStrm = strmConnection.openInputStream();

                System.out.println("serviceSearchCompleted");
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }

        }
    }

    /**Cierra los streams*/
    void CloseAll()
    {
        try
        {
            if(outputStrm!=null)
                outputStrm.close();//cierra el stream de salida

            if( inputStrm!=null)
                 inputStrm.close();//cierra el stream de entrada
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }

    }



    /**Llamado cuando se encuentra un servicio durante la busqueda*/
    public void servicesDiscovered(int transID, ServiceRecord[] records)
    {

        for (int i = 0; i < records.length; i++)
        {
            strUrl = records[i].getConnectionURL(ServiceRecord.AUTHENTICATE_ENCRYPT, false);

            System.out.println(strUrl);
            if(strUrl.startsWith("btspp")) //se encontro un servicio con el protocolo correcto
            {
                serverFound = true;
                initClient=true;
               break;
            }

        }
    }

    /**Llamado cuando se encutro un dispositivo durante el servicio de investigacion
    @param btDevicen dispositivo remoto bluetooth
    @param cod codigo del dipositivo*/
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod)
    {

        try
        {
            //Obtiene la informacion del dispositivo y la imprime para debugeo
            System.out.println("Device Discovered");
            System.out.println("Major Device Class: " + cod.getMajorDeviceClass() + " Minor Device Class: " + cod.getMinorDeviceClass());
            System.out.println("Bluetooth Address: " + btDevice.getBluetoothAddress());
            System.out.println("Bluetooth Friendly Name: " + btDevice.getFriendlyName(true));

            //busca por servicios
            UUID uuidSet[] = new UUID[1];
            uuidSet[0] =  RFCOMM_UUID;
            int searchID = discovAgent.searchServices(null,uuidSet,btDevice,this);
        }
        catch (Exception e)
        {
            System.out.println("Device Discovered Error: " + e);
        }


    }

}
